/*
 * Copyright 2009 Nicolas LE GRIEL.
 * 
 * Les sources qui constituent ce projet nlegriel-dvp-defi5 de mme que 
 * la documentation associe sont la proprit de leur auteur.
 * 
 * Je donne mon accord au site developpez.com pour l'utilisation de 
 * tout ou partie des sources et de la documentation de ce projet dans 
 * les pages developpez.com
 */
package com.developpez.nlegriel.defi5;

import static com.developpez.nlegriel.defi5.IntegerMatrix3x3.ALL_ROTATIONS;

/**
 * Classe utilitaire fournissant toutes les orientations possibles pour
 * les sept pices du cube de Soma.
 * <p>
 * Pour chaque pice, on choisi un cube de coordonnes (0, 0, 0).
 * On fait subir ensuite  cette pice l'ensemble des vingt-quatre
 * isomtries positives laissant le cube invariant. Cet ensemble est 
 * gnr  partir du systme de rotations suivant :
 * <ul>
 * 	<li><i>X</i> rotation d'axe l'axe des abscisses et d'angle pi/2</li>
 * 	<li><i>Y</i> rotation d'axe l'axe des ordonnes et d'angle pi/2</li>
 * 	<li><i>Z</i> rotation d'axe l'axe des cotes et d'angle pi/2</li>
 * </ul>
 * Pour obtenir toutes les rotations, on applique le principe suivant : on
 * considre un cube et on lui applique six rotations qui permettent de "voir"
 * chacune de ses six faces. Ici, j'ai choisi de regarder le point de 
 * coordonnes (0, 0, 0) depuis l'axe des abscisses et de "voir" les six
 * faces d'un cube centre en (0, 0, 0)  l'aide des rotations <i>I</i> (pour
 * l'identit), <i>Z</i>, <i>ZZ</i> (compose de <i>Z</i> par <i>Z</i>),
 * <i>ZZZ</i>, <i>Y</i> et <i>YY</i>. Ensuite, pour chaque face qui se
 * "prsente  moi", je lui applique quatre rotations autour de l'axe des
 * abscisses, c'est--dire que je dois composer chacune des six rotations 
 * prsentes ci-dessus par les rotations suivantes : <i>I</i>, <i>X</i>,
 * <i>XX</i> et <i>XXX</i>.
 *
 * @author Nicolas LE GRIEL
 */
public class PuzzlePartPositionsProvider {
	/**
	 * Calcule  partir des coordones des cubes composant une pice du
	 * puzzle l'ensemble des positions pouvant prendre cette pice tout en
	 * passant par le cube de coordonnes (0, 0, 0).
	 * 
	 * @param partCubes cubes constituant la pice  tudier, il doit
	 * obligatoirement passer par le cube de coordonnes (0, 0, 0)
	 * 
	 * @return un tableau  deux dimensions donnant toutes les compositions de 
	 * translation par une rotation faisant passer la pice par la coordonne 
	 * (0, 0, 0). Le premier indice fournit les compositions et le second 
	 * indice indique les positions rsultantes des cubes de la pice par
	 * l'isomtrie concerne.
	 */
	public static Cube[][] getPositionsOfPart(Cube[] partCubes) {
		Cube[][] result;
		result = new Cube[ALL_ROTATIONS.size() * partCubes.length][partCubes.length];
		
		Cube[][] translations = getTranslationsOfPart(partCubes);
		
		int t = translations.length;
		int r = ALL_ROTATIONS.size();
		int p = partCubes.length;
		for (int i=0; i<t; i++)
			for (int j=0; j<r; j++)
				for (int k=0; k<p; k++)
					result[i*r + j][k] = ALL_ROTATIONS.get(j).operate(translations[i][k]);
		
		return result;
	}

	/**
	 * Calcule pour un ensemble de cubes donn passant par le cube de coordonnes
	 * (0, 0, 0) les translations transformant chacune un des cubes de 
	 * l'ensemble de dpart en le cube de coordonnes (0, 0, 0).
	 * 
	 * @param partCubes ensemble de cubes pour lequel calculer les translations,
	 * cet ensemble devant obligatoirement contenir le cube de coordonnes
	 * (0, 0, 0)
	 * 
	 * @return un tableau regroupant pour chaque valeur du premier indice, les
	 * positions rsultantes des cubes de l'ensemble initial par les translations
	 * recherches. La translation nulle faisant partie de ces translations, le
	 * paramtre <code>partCubes</code> fait partie du rsultat.
	 */
	protected static Cube[][] getTranslationsOfPart(Cube[] partCubes) {
		Cube[][] result = new Cube[partCubes.length][partCubes.length];
		
		// Rsultat de la translation nulle :
		int p = partCubes.length;
		for(int i=0; i<p; i++)
			result[0][i] = partCubes[i];
		
		// Application des autres translations :
		for(int i=1; i<p; i++) {
			Cube translation = new Cube(-partCubes[i].x, -partCubes[i].y, -partCubes[i].z);
			for (int j=0; j<p; j++)
				result[i][j] = partCubes[j].translate(translation);
		}
		
		return result;
	}
}
