/*
 * Copyright 2009 Nicolas LE GRIEL.
 * 
 * Les sources qui constituent ce projet nlegriel-dvp-defi5 de mme que 
 * la documentation associe sont la proprit de leur auteur.
 * 
 * Je donne mon accord au site developpez.com pour l'utilisation de 
 * tout ou partie des sources et de la documentation de ce projet dans 
 * les pages developpez.com
 */
package com.developpez.nlegriel.defi5.view;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GraphicsConfiguration;
import java.awt.Toolkit;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.Material;
import javax.media.j3d.PointLight;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3d;

import com.developpez.nlegriel.defi5.Cube;
import com.developpez.nlegriel.defi5.CubeCover;
import com.developpez.nlegriel.defi5.Puzzle;
import com.developpez.nlegriel.defi5.PuzzlePart;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.universe.SimpleUniverse;

/**
 * Fentre permettant de visualiser les lments d'un puzzle ainsi que
 * des placements dans l'espace des pices du cube de Soma.
 *
 * @author Nicolas LE GRIEL
 */
public class CubesViewWindow extends JFrame {
	private static final long serialVersionUID = 1L;

	/**
	 * Map immutable utilise pour grer la coloration des pices du
	 * puzzle lors de l'affichage d'une solution.
	 */
	private final static Map<PuzzlePart, Color3f> partColors;
	static {
		Map<PuzzlePart, Color3f> colors = new HashMap<PuzzlePart, Color3f>(7);
		colors.put(PuzzlePart.A, new Color3f(1, 0, 0));
		colors.put(PuzzlePart.B, new Color3f(0, 1, 0));
		colors.put(PuzzlePart.V, new Color3f(0, 0, 1));
		colors.put(PuzzlePart.L, new Color3f(1, 1, 1));
		colors.put(PuzzlePart.T, new Color3f(1, 1, 0));
		colors.put(PuzzlePart.Z, new Color3f(1, 0, 1));
		colors.put(PuzzlePart.P, new Color3f(0, 1, 1));
		partColors = Collections.unmodifiableMap(colors);
	}
	
	private Puzzle puzzle;
	
	private JPanel contentPanel;
	private Canvas3D canvas3D;
	

	public CubesViewWindow(String title, Puzzle puzzle) {
		super(title);
		this.puzzle = puzzle;
		initialize();
	}

	/**
	 * Gre toute la construction des lments de la fentre.
	 */
	private void initialize() {
		setContentPane(getContentPanel());
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		pack();
		setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - getWidth()) / 2, (Toolkit.getDefaultToolkit().getScreenSize().height - getHeight()) / 2);
		setVisible(true);	}

	public Container getContentPanel() {
		if (contentPanel == null) {
			contentPanel = new JPanel();
			contentPanel.setLayout(new BorderLayout());
			contentPanel.add(getCanvas3D(), BorderLayout.CENTER);
		}
		return contentPanel;
	}
	
	public Canvas3D getCanvas3D() {
		if (canvas3D == null) {
			GraphicsConfiguration graphicsConfiguration = SimpleUniverse.getPreferredConfiguration();
			canvas3D = new Canvas3D(graphicsConfiguration);
			canvas3D.setSize(500, 500);
			
			SimpleUniverse universe = new SimpleUniverse(canvas3D);

			// Cration du noeud racine et de la matrice de transformation de la branche volume :
			TransformGroup tgVolume = new TransformGroup();
			BranchGroup rootVolume = new BranchGroup();
			rootVolume.addChild(tgVolume);
			
			// Placer la camra :
			Transform3D eye = new Transform3D();
			eye.setTranslation(new Vector3d(0, 0, MoveBehavior.INITIAL_EYE_COTE));
			universe.getViewingPlatform().setNominalViewingTransform();
			universe.getViewingPlatform().getViewPlatformTransform().setTransform(eye);
			
			// Permettre le mouvement de la camra :
			MoveBehavior viewMove = new MoveBehavior(universe.getViewingPlatform().getViewPlatformTransform());
			viewMove.setSchedulingBounds(new BoundingSphere(new Point3d(0, 0, 0), 500));
			rootVolume.addChild(viewMove);
			
			// Rcuprer les lments qui vont permettre de dessiner et dplacer les cubes :
			tgVolume.addChild(getCubesBranchGroup());
			universe.addBranchGraph(rootVolume);
		}
		return canvas3D;
	}
	
	/**
	 * Dessine l'ensemble des cubes de la figure recherche ainsi que les 
	 * ventuelles pices places sur le puzzle.
	 * 
	 * @return lments Java3D indiquant les positions des cubes et pices du
	 * puzzle
	 */
	private BranchGroup getCubesBranchGroup() {
		BranchGroup result = new BranchGroup();

		// Un peu de lumire :
		Color3f lightColor = new Color3f(1, 1, 1);
		PointLight pointLight = new PointLight(lightColor, new Point3f(25, 5, 5), new Point3f(1, 0, 0));
		pointLight.setInfluencingBounds(new BoundingSphere(new Point3d(0, 0, 0), 100));
		result.addChild(pointLight);

		// Dessiner les cubes non pris de la figure :
		for (Cube cube : puzzle.uncoveredCubes)
			result.addChild(createCube(cube));

		// Dessiner les pices places du puzzle :
		for (CubeCover cube : puzzle.coveredCubes)
			result.addChild(createCube(cube.cube, partColors.get(cube.part)));
		
		result.compile();
		return result;
	}
	
	/**
	 * Dessinne un cube du puzzle non pris par une des
	 * pices du puzzle
	 * 
	 * @param cube contient les coordonne du cube  dessiner
	 * 
	 * @return les lments gomtriques sur le cube  dessiner
	 */
	private TransformGroup createCube(Cube cube) {
		return createCube(cube, new Color3f(.55f, .55f, .55f));
	}

	/**
	 * Dessine un cube avec une couleur afin que l'on puisse distinguer
	 * les cubes de la figure et/ou les cubes de chaque pice du puzzle.
	 * 
	 * @param cube coordonnes du cube  tracer
	 * @param color couleur du cube  dessiner
	 * 
	 * @return informations pour que Java3D puisse produire le rendu du cube
	 */
	private TransformGroup createCube(Cube cube, Color3f color) {
		// Couleur du cube :
		Appearance appearance = new Appearance();
		Material material = new Material();
		material.setDiffuseColor(color);
		material.setAmbientColor(color);
		material.setEmissiveColor(new Color3f(color.x*3/4, color.y*3/4, color.z*3/4));
		material.setShininess(60);
		appearance.setMaterial(material);

		// Position du cube :
		TransformGroup result = new TransformGroup();
		Transform3D cubePosition = new Transform3D();
		cubePosition.setTranslation(new Vector3d(cube.x, cube.y, cube.z));
		result.setTransform(cubePosition);
		
		/*
		 *  Association entre les caractristiques du cube.
		 *  Les cubes sont lgrement sous-dimensionns afin que l'on
		 *  puisse les distinguer les uns des autres (petit espace laiss
		 *  entre les cubes).
		 */
		Box box = new Box(.48f, .48f, .48f, appearance);
		result.addChild(box);
		return result;
	}
}
